DefaultMouth

Struct DefaultMouth 

pub struct DefaultMouth<ET: EngineTypes> { /* private fields */ }
The default implementation of Mouth. Well optimized to be fast, but at the cost of not keeping track of “the depth of current macro expansions” - I found that to be an acceptable loss, since afaict, the only thing that is done with that information is to print a corresponding number of .s if \tracingcommands is set. By omitting that and just concatenating all expansions into a single Vec from which to pop(), we gain a massive speedup.

Errors

Implementations

Source

impl<ET: EngineTypes> DefaultMouth<ET>

pub fn into<ET2: EngineTypes<Token = ET::Token, File = ET::File>>( self, ) -> DefaultMouth<ET2>

Trivial conversion between different compatible EngineTypes.

pub fn into_tokens<ET2: EngineTypes<Char = ET::Char, File = ET::File>, F: FnMut(ET::Token) -> ET2::Token>( self, token: F, ) -> DefaultMouth<ET2>

Less trivial conversion between different EngineTypes with compatible Tokens.